@@ -63,7 +63,7 @@ use crate::{
 ///
 /// As such, we encourage not to use `ChatId::ChannelUsername(u)` with this bot
 /// wrapper.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct Throttle<B> {
     bot: B,
     // `RequestLock` allows to unlock requests (allowing them to be sent).
@@ -203,11 +203,15 @@ pub struct Limits {
     /// Allowed messages in one chat per minute.
     pub messages_per_min_chat: u32,
 
+    /// Allowed messages in one channel per minute.
+    pub messages_per_min_channel: u32,
+
     /// Allowed messages per second.
     pub messages_per_sec_overall: u32,
 }
 
-/// Defaults are taken from [telegram documentation][tgdoc].
+/// Defaults are taken from [telegram documentation][tgdoc] (except for
+/// `messages_per_min_channel`).
 ///
 /// [tgdoc]: https://core.telegram.org/bots/faq#my-bot-is-hitting-limits-how-do-i-avoid-this
 impl Default for Limits {
@@ -216,6 +220,7 @@ impl Default for Limits {
             messages_per_sec_chat: 1,
             messages_per_sec_overall: 30,
             messages_per_min_chat: 20,
+            messages_per_min_channel: 10,
         }
     }
 }
@@ -465,9 +470,17 @@ async fn worker(
 
         while let Some(entry) = queue_removing.next() {
             let chat = &entry.value().0;
-            let requests_sent_count = requests_sent.per_sec.get(chat).copied().unwrap_or(0);
-            let limits_not_exceeded = requests_sent_count < limits.messages_per_sec_chat
-                && requests_sent_count < limits.messages_per_min_chat;
+            let requests_sent_per_sec_count = requests_sent.per_sec.get(chat).copied().unwrap_or(0);
+            let requests_sent_per_min_count = requests_sent.per_min.get(chat).copied().unwrap_or(0);
+
+            let messages_per_min_limit = if chat.is_channel() {
+                limits.messages_per_min_channel
+            } else {
+                limits.messages_per_min_chat
+            };
+
+            let limits_not_exceeded = requests_sent_per_sec_count < limits.messages_per_sec_chat
+                && requests_sent_per_min_count < messages_per_min_limit;
 
             if limits_not_exceeded {
                 *requests_sent.per_sec.entry(*chat).or_insert(0) += 1;
@@ -495,7 +508,7 @@ async fn worker(
 
 fn answer_info(rx: &mut mpsc::Receiver<InfoMessage>, limits: &mut Limits) {
     // FIXME(waffle): https://github.com/tokio-rs/tokio/issues/3350
-    while let Some(Some(req)) = rx.recv().now_or_never() {
+    while let Some(Some(req)) = tokio::task::unconstrained(rx.recv()).now_or_never() {
         // Errors are ignored with .ok(). Error means that the response channel
         // is closed and the response isn't needed.
         match req {
@@ -519,7 +532,7 @@ async fn read_from_rx<T>(rx: &mut mpsc::Receiver<T>, queue: &mut Vec<T>, rx_is_c
     // Don't grow queue bigger than the capacity to limit DOS posibility
     while queue.len() < queue.capacity() {
         // FIXME(waffle): https://github.com/tokio-rs/tokio/issues/3350
-        match rx.recv().now_or_never() {
+        match tokio::task::unconstrained(rx.recv()).now_or_never() {
             Some(Some(req)) => queue.push(req),
             Some(None) => *rx_is_closed = true,
             // There are no items in queue.
@@ -592,7 +605,8 @@ where
         stop_message_live_location, stop_message_live_location_inline,
         send_chat_action, get_user_profile_photos, get_file, kick_chat_member, ban_chat_member,
         unban_chat_member, restrict_chat_member, promote_chat_member,
-        set_chat_administrator_custom_title, set_chat_permissions,
+        set_chat_administrator_custom_title,
+        ban_chat_sender_chat, unban_chat_sender_chat, set_chat_permissions,
         export_chat_invite_link, create_chat_invite_link, edit_chat_invite_link,
         revoke_chat_invite_link, set_chat_photo, delete_chat_photo, set_chat_title,
         set_chat_description, pin_chat_message, unpin_chat_message, unpin_all_chat_messages,
@@ -606,7 +620,8 @@ where
         add_sticker_to_set, set_sticker_position_in_set, delete_sticker_from_set,
         set_sticker_set_thumb, answer_shipping_query, answer_pre_checkout_query,
         set_passport_data_errors, send_game, set_game_score, set_game_score_inline,
-        get_game_high_scores, get_updates_fault_tolerant => fid, ftyid
+        approve_chat_join_request, decline_chat_join_request,
+        get_game_high_scores => fid, ftyid
     }
 }
 
@@ -627,6 +642,15 @@ enum ChatIdHash {
     ChannelUsernameHash(u64),
 }
 
+impl ChatIdHash {
+    fn is_channel(&self) -> bool {
+        match self {
+            &Self::Id(id) => ChatId::Id(id).is_channel(),
+            Self::ChannelUsernameHash(_) => true,
+        }
+    }
+}
+
 impl From<&ChatId> for ChatIdHash {
     fn from(value: &ChatId) -> Self {
         match value {
@@ -641,6 +665,7 @@ impl From<&ChatId> for ChatIdHash {
     }
 }
 
+#[must_use = "Requests are lazy and do nothing unless sent"]
 pub struct ThrottlingRequest<R: HasPayload> {
     request: R,
     chat_id: fn(&R::Payload) -> ChatIdHash,
