@@ -1,41 +1,3 @@
-macro_rules! forward_to_unsuported_ty {
-    (
-        supported: $supported:expr;
-        simple { $( $method:ident $arg:ty )* }
-        unit { $( $method1:ident $ty:expr )* }
-        compound {
-            $( $method2:ident $( <$T:ident: ?Sized + Serialize> )? ( $( $args:tt )* ) -> $ret:ty => $message:expr )*
-        }
-    ) => {
-        $(
-            fn $method(self, _: $arg) -> Result<Self::Ok, Self::Error> {
-                Err(Self::Error::UnsupportedType {
-                    ty: stringify!($arg),
-                    supported: $supported,
-                })
-            }
-        )+
-
-        $(
-            fn $method1(self) -> Result<Self::Ok, Self::Error> {
-                Err(Self::Error::UnsupportedType {
-                    ty: $ty,
-                    supported: $supported,
-                })
-            }
-        )+
-
-        $(
-            fn $method2 $( <$T: ?Sized + Serialize> )? (self, $( $args )*) -> Result<$ret, Self::Error> {
-                Err(Self::Error::UnsupportedType {
-                    ty: $message,
-                    supported: $supported,
-                })
-            }
-        )+
-    };
-}
-
 macro_rules! req_future {
     (
         $v2:vis def: | $( $arg:ident: $ArgTy:ty ),* $(,)? | $body:block
@@ -135,7 +97,7 @@ macro_rules! calculated_doc {
 macro_rules! impl_payload {
     (
         $(
-            @[$multipart_attr:ident]
+            @[multipart = $($multipart_attr:ident),*]
         )?
         $(
             #[ $($method_meta:tt)* ]
@@ -241,7 +203,7 @@ macro_rules! impl_payload {
 
         impl<P> $Setters for P where P: crate::requests::HasPayload<Payload = $Method> {}
 
-        impl_payload! { @[$($multipart_attr)?] $Method req { $($($fields),*)? } opt { $($($opt_fields),*)? } }
+        impl_payload! { @[$(multipart = $($multipart_attr),*)?] $Method req { $($($fields),*)? } opt { $($($opt_fields),*)? } }
     };
     (@setter_opt $Method:ident $field:ident : $FTy:ty [into]) => {
         calculated_doc! {
@@ -255,6 +217,7 @@ macro_rules! impl_payload {
                 ") field."
             )]
             #[allow(clippy::wrong_self_convention)]
+            #[must_use = "Payloads and requests do nothing unless sent"]
             fn $field<T>(mut self, value: T) -> Self
             where
                 T: Into<$FTy>,
@@ -276,6 +239,7 @@ macro_rules! impl_payload {
                 ") field."
             )]
             #[allow(clippy::wrong_self_convention)]
+            #[must_use = "Payloads and requests do nothing unless sent"]
             fn $field<T>(mut self, value: T) -> Self
             where
                 T: ::core::iter::IntoIterator<Item = <$FTy as ::core::iter::IntoIterator>::Item>,
@@ -297,6 +261,7 @@ macro_rules! impl_payload {
                 ") field."
             )]
             #[allow(clippy::wrong_self_convention)]
+            #[must_use = "Payloads and requests do nothing unless sent"]
             fn $field(mut self, value: $FTy) -> Self {
                 self.payload_mut().$field = Some(value);
                 self
@@ -315,6 +280,7 @@ macro_rules! impl_payload {
                 ") field."
             )]
             #[allow(clippy::wrong_self_convention)]
+            #[must_use = "Payloads and requests do nothing unless sent"]
             fn $field<T>(mut self, value: T) -> Self
             where
                 T: Into<$FTy>,
@@ -336,6 +302,7 @@ macro_rules! impl_payload {
                 ") field."
             )]
             #[allow(clippy::wrong_self_convention)]
+            #[must_use = "Payloads and requests do nothing unless sent"]
             fn $field<T>(mut self, value: T) -> Self
             where
                 T: ::core::iter::IntoIterator<Item = <$FTy as ::core::iter::IntoIterator>::Item>,
@@ -357,6 +324,7 @@ macro_rules! impl_payload {
                 ") field."
             )]
             #[allow(clippy::wrong_self_convention)]
+            #[must_use = "Payloads and requests do nothing unless sent"]
             fn $field(mut self, value: $FTy) -> Self {
                 self.payload_mut().$field = value;
                 self
@@ -381,9 +349,20 @@ macro_rules! impl_payload {
     (@convert_map ($e:expr)) => {
         $e
     };
-    (@[multipart] $Method:ident req { $($reqf:ident),* } opt { $($optf:ident),*} ) => {
-        impl crate::requests::MultipartPayload for $Method {}
-        impl crate::requests::multipart_payload::sealed::Sealed for $Method {}
+    (@[multipart = $($multipart_attr:ident),*] $Method:ident req { $($reqf:ident),* } opt { $($optf:ident),*} ) => {
+        impl crate::requests::MultipartPayload for $Method {
+            fn copy_files(&self, into: &mut dyn FnMut(crate::types::InputFile)) {
+                $(
+                    crate::types::InputFileLike::copy_into(&self.$multipart_attr, into);
+                )*
+            }
+
+            fn move_files(&mut self, into: &mut dyn FnMut(crate::types::InputFile)) {
+                $(
+                    crate::types::InputFileLike::move_into(&mut self.$multipart_attr, into);
+                )*
+            }
+        }
     };
     (@[] $($ignored:tt)*) => {}
 }
@@ -645,11 +624,11 @@ macro_rules! requester_forward {
     (@method send_poll $body:ident $ty:ident) => {
         type SendPoll = $ty![SendPoll];
 
-        fn send_poll<C, Q, O>(&self, chat_id: C, question: Q, options: O, type_: PollType) -> Self::SendPoll where C: Into<ChatId>,
+        fn send_poll<C, Q, O>(&self, chat_id: C, question: Q, options: O) -> Self::SendPoll where C: Into<ChatId>,
         Q: Into<String>,
         O: IntoIterator<Item = String> {
             let this = self;
-            $body!(send_poll this (chat_id: C, question: Q, options: O, type_: PollType))
+            $body!(send_poll this (chat_id: C, question: Q, options: O))
         }
     };
     (@method send_dice $body:ident $ty:ident) => {
@@ -733,6 +712,22 @@ macro_rules! requester_forward {
             $body!(set_chat_administrator_custom_title this (chat_id: Ch, user_id: i64, custom_title: Cu))
         }
     };
+    (@method ban_chat_sender_chat $body:ident $ty:ident) => {
+        type BanChatSenderChat = $ty![BanChatSenderChat];
+
+        fn ban_chat_sender_chat<C>(&self, chat_id: C, sender_chat_id: i64) -> Self::BanChatSenderChat where C: Into<ChatId> {
+            let this = self;
+            $body!(ban_chat_sender_chat this (chat_id: C, sender_chat_id: i64))
+        }
+    };
+    (@method unban_chat_sender_chat $body:ident $ty:ident) => {
+        type UnbanChatSenderChat = $ty![UnbanChatSenderChat];
+
+        fn unban_chat_sender_chat<C>(&self, chat_id: C, sender_chat_id: i64) -> Self::UnbanChatSenderChat where C: Into<ChatId> {
+            let this = self;
+            $body!(unban_chat_sender_chat this (chat_id: C, sender_chat_id: i64))
+        }
+    };
     (@method set_chat_permissions $body:ident $ty:ident) => {
         type SetChatPermissions = $ty![SetChatPermissions];
 
@@ -775,6 +770,22 @@ macro_rules! requester_forward {
             $body!(revoke_chat_invite_link this (chat_id: C, invite_link: I))
         }
     };
+    (@method approve_chat_join_request $body:ident $ty:ident) => {
+        type ApproveChatJoinRequest = $ty![ApproveChatJoinRequest];
+
+        fn approve_chat_join_request<C>(&self, chat_id: C, user_id: i64) -> Self::ApproveChatJoinRequest where C: Into<ChatId> {
+            let this = self;
+            $body!(approve_chat_join_request this (chat_id: C, user_id: i64))
+        }
+    };
+    (@method decline_chat_join_request $body:ident $ty:ident) => {
+        type DeclineChatJoinRequest = $ty![DeclineChatJoinRequest];
+
+        fn decline_chat_join_request<C>(&self, chat_id: C, user_id: i64) -> Self::DeclineChatJoinRequest where C: Into<ChatId> {
+            let this = self;
+            $body!(decline_chat_join_request this (chat_id: C, user_id: i64))
+        }
+    };
     (@method set_chat_photo $body:ident $ty:ident) => {
         type SetChatPhoto = $ty![SetChatPhoto];
 
@@ -1157,12 +1168,4 @@ macro_rules! requester_forward {
             $body!(get_game_high_scores this (user_id: i64, target: T))
         }
     };
-    (@method get_updates_fault_tolerant $body:ident $ty:ident) => {
-        type GetUpdatesFaultTolerant = $ty![GetUpdatesFaultTolerant];
-
-        fn get_updates_fault_tolerant(&self) -> Self::GetUpdatesFaultTolerant {
-            let this = self;
-            $body!(get_updates_fault_tolerant this ())
-        }
-    };
 }
