@@ -1,3 +1,5 @@
+use either::Either;
+
 use crate::requests::Payload;
 
 /// Represents types having payload inside.
@@ -42,3 +44,19 @@ where
         self
     }
 }
+
+impl<L, R> HasPayload for Either<L, R>
+where
+    L: HasPayload,
+    R: HasPayload<Payload = L::Payload>,
+{
+    type Payload = L::Payload;
+
+    fn payload_mut(&mut self) -> &mut Self::Payload {
+        self.as_mut().either(<_>::payload_mut, <_>::payload_mut)
+    }
+
+    fn payload_ref(&self) -> &Self::Payload {
+        self.as_ref().either(<_>::payload_ref, <_>::payload_ref)
+    }
+}
